#include "toolbox.h"
#include <formatio.h>
#include "rsspecan.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsspecan_hardcopy_example.h"


#define ARRAY_SIZE    5000
#define STRING_LENGTH 512
#define WINDOW        1

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
volatile ViBoolean  enableTrace  = VI_FALSE; 
static   ViSession  instrSession = VI_NULL;
ViChar   filePath[STRING_LENGTH] = {NULL};
ViString tempFilePath            = "";
    

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);
int readData(void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsspecan_hardcopy_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument and initiates the measurement
/*===========================================================================*/
int CVICALLBACK saveButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
            {
            
                /* block mouse events */
                SetWaitCursor (1);
				tempFilePath = "C:\\R_S\\instr\\user\\output.wmf";
                
                /* read values from panel*/
                GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
                GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
                GetCtrlVal (panelHandle, PANEL_RESET, &reset);
                GetCtrlVal (panelHandle, PANEL_DISPLAYON, &display);
                
                /* disable buttons for configuration */
                EnableButtons(VI_FALSE);
                
                /* initialize driver */
                CHECKERR (rsspecan_init (instr_desc, id, reset, &instrSession));
                
                /* switch device display on/off */
                CHECKERR (rsspecan_ConfigureDisplayUpdate(instrSession, display));
                
                /* Set instrument to single sweep mode - continous mode is done
                 * programmatically because this is the only way how to
                 * synchronize measurement using *OPC
                 */
                CHECKERR (rsspecan_ConfigureAcquisition (instrSession, 0, VI_FALSE, 1));
				
                /* configure hardcopy options */
                CHECKERR (rsspecan_ConfigureHardcopyDevice (instrSession,1, RSSPECAN_VAL_HCOPY_DEST_WMF, RSSPECAN_VAL_HCOPY_DEVICE_ORIENT_LAND));
                
                /* configure hardcopy save file */
                CHECKERR (rsspecan_HardcopySetFileName (instrSession, tempFilePath));
                
                /* save screen image to file */ 
                CHECKERR (rsspecan_HardcopyPrint (instrSession, 1, RSSPECAN_VAL_HCOP_ALL));
                
                /* read data and save to file */
                readData();
                
                /* close session */
                cleanup(VI_FALSE);
            }
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Open Button Callback
/* Purpose:  Opens file for saving measurement results
/*===========================================================================*/
int  CVICALLBACK openButton(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
         
    {
        case EVENT_COMMIT:
            
            /* open or create new file */
            FileSelectPopup ("", "*.wmf", "*.wmf", "Name of File to Save",
                             VAL_OK_BUTTON, 0, 1, 1, 1, filePath);
            
            /* write file path to front panel text box */
            SetCtrlVal (panelHandle, PANEL_FILEPATH, filePath);
    }
    return 0;
}

/*===========================================================================*/
/* Function: Read Data
/* Purpose:  Reads file from the instrument and saves it into specified file
/*===========================================================================*/
int readData (void)
{
    ViStatus status       = VI_SUCCESS;
    
    /* Is save file specified? */
    GetCtrlVal (panelHandle, PANEL_FILEPATH, filePath);
    if ((filePath[0] == NULL) | (!FileExists(filePath,0)))
    {
        /* open or create new file or exit if canceled */
        if (FileSelectPopup ("", "*.wmf", "*.wmf", "Name of File to Save",
                             VAL_OK_BUTTON, 0, 1, 1, 1, filePath) == 0)
            return -1;
            
        /* write file path to front panel text box */
        SetCtrlVal (panelHandle, PANEL_FILEPATH, filePath);
    }
    
    /* copy file from instrument to controler */
    CHECKERR (rsspecan_ReadToFileFromInstrument (instrSession, tempFilePath, filePath));
    
    /* delete temporary file */
    CHECKERR (rsspecan_FileManagerOperations (instrSession, RSSPECAN_VAL_FILE_DELETE, tempFilePath, ""));
    
    return 0;
}


/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsspecan_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsspecan_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_SAVE,      ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,     ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,   ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_DISPLAYON, ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_FILEPATH,  ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_OPEN,      ATTR_DIMMED,!enable);
    
    
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsspecan_close (instrSession);
    instrSession = VI_NULL;
    
    /* no measurements */
    enableTrace  = VI_FALSE;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}


